'use strict';

const opentelemetry_api = require('@opentelemetry/api');
const { ROOT_CONTEXT } = require('@opentelemetry/api');
const { BatchSpanProcessor, ConsoleSpanExporter, BasicTracerProvider } = require('@opentelemetry/tracing');
const { CollectorTraceExporter } = require("@opentelemetry/exporter-collector");
const { AsyncHooksContextManager, AsyncLocalStorageContextManager } = require("@opentelemetry/context-async-hooks");
const { ParentBasedSampler, AlwaysOnSampler } = require("@opentelemetry/core");
const { Resource } = require('@opentelemetry/resources');
const { SemanticResourceAttributes } = require('@opentelemetry/semantic-conventions');
const url = require('url');

module.exports = TracerProvider;

function TracerProvider(logger) {
  this.host = 'localhost';
  this.port = '55681';
  this.ot_api = opentelemetry_api;
  this.ROOT_CONTEXT = ROOT_CONTEXT;
  this.logger = logger;
}

TracerProvider.prototype.register = function(config) {
  const provider = new BasicTracerProvider({
    sampler: new ParentBasedSampler({
      root: new AlwaysOnSampler()
    }),
    resource: new Resource({
      [SemanticResourceAttributes.SERVICE_NAME]: config.tierName,
      "service.name": config.tierName,
      "service.namespance": config.applicationName
    }),
  });

  // default collector configuration, can be overridden from agent config
  const defaultOptions = {
    attributes: {'service.name': config.tierName,
                 'service.namespace': config.applicationName}
  };

  if (config.openTelemetry && config.openTelemetry.collector) {
    Object.assign(defaultOptions, config.openTelemetry.collector);
    this.url = defaultOptions.url;
    try {
      var urlObj = url.parse(this.url);
      this.host = urlObj.hostname;
      this.port = urlObj.port;
    } catch (e) {
      this.logger.error('Collector url must be in <host>:<port> format');
      return false;
    }
  }

  const exporter = new CollectorTraceExporter(defaultOptions);
  provider.addSpanProcessor(new BatchSpanProcessor(exporter));
  if (config.openTelemetry && config.openTelemetry.debug) {
    provider.addSpanProcessor(new BatchSpanProcessor(new ConsoleSpanExporter()));
  }

  const majorVersion = parseInt(process.versions.node.split('.')[0]);
  const minorVersion = parseInt(process.versions.node.split('.')[1]);
  const contextManager = majorVersion > 14 || majorVersion == 14 && minorVersion >= 8 ?
    AsyncLocalStorageContextManager : AsyncHooksContextManager;
  provider.register({contextManager: new contextManager()});
  return true;
};

TracerProvider.prototype.getTracer = function(name) {
  return opentelemetry_api.trace.getTracer(name);
};